==> fap-0.8.1.tar.bz2: fap-0.8.1/src/gui/infoDialog.cpp <==
/*
 *  Copyright (C) 2006-2010 Leandro Nini
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
 */

#include "infoDialog.h"
#include "textLabel.h"
#include "settings.h"
#include "iconFactory.h"
#include "utils.h"

#if FOX_LEVEL<32
#  include <FXJPGImage.h>
#endif

#define IMAGESIZE 150

FXint imageLoader::run() {

	FXIconSource iconsource(FXApp::instance());
	FXImage *img;

#if FOX_LEVEL<32
	// Hack to overcome a FOX bug
	if (FXJPGImage::supported && !comparecase(FXPath::extension(_name), "jpeg"))
		img=iconsource.loadScaledImageFile(_name, IMAGESIZE, 0, FXJPGImage::fileExt);
	else
#endif
	img=iconsource.loadScaledImageFile(_name, IMAGESIZE, SETTINGS->albumArtQuality());

	_loaded->setData((void*)(FXival)img);
	_loaded->signal();

	return 0;
}

FXbool imageLoader::load(const FXString& file) {

	_name=file;

	return start();
}

/*****************************************************************/

FXDEFMAP(infoDialog) infoDialogMap[] = {
	// Message_Type			ID				Message_Handler
	FXMAPFUNC(SEL_IO_READ,		infoDialog::ID_IMG_LOADED,	infoDialog::onCmdImgLoaded),
};

FXIMPLEMENT(infoDialog, FXDialogBox, infoDialogMap, ARRAYNUMBER(infoDialogMap))

infoDialog::infoDialog(FXWindow* win, const metaData* i) :
	FXDialogBox(win, FXString::null, DECOR_TITLE|DECOR_BORDER),
	_font(0),
	_imageLoader(0),
	_imgFrame(0) {

	setTitle(tr("Song Info"));

	new FXButton(this, tr("&Close"), GET_ICON(icon_dialogcancel), this, FXDialogBox::ID_CANCEL,
		BUTTON_INITIAL|BUTTON_DEFAULT|BUTTON_NORMAL|LAYOUT_CENTER_X|LAYOUT_SIDE_BOTTOM);
	new FXSeparator(this, SEPARATOR_GROOVE|LAYOUT_FILL_X|LAYOUT_SIDE_BOTTOM);

	FXString folder=i->getInfo(metaData::LOCATION);
	if (folder.empty()) {
		new FXLabel(this, tr("No song loaded"));
		return;
	}

	FXHorizontalFrame *container=new FXHorizontalFrame(this, LAYOUT_FILL_X, 0,0,0,0, 0,0,0,0, 0,0);

	_imgFrame=new FXImageFrame(container, 0, LAYOUT_FIX_WIDTH|LAYOUT_FIX_HEIGHT|FRAME_SUNKEN|FRAME_THICK, 0, 0, IMAGESIZE, IMAGESIZE);

	FXMatrix* matrix=new FXMatrix(container, 2, MATRIX_BY_COLUMNS|LAYOUT_FILL_X);

	FXColor lblColor=altColor(getApp()->getForeColor());

	int j=-1;
	while ((j=i->moreInfo(j))>=0) {
		FXString temp=i->getInfo(j);
		if (temp.empty())
			continue;

		unsigned int rows=1;
		unsigned int cols=0;
		unsigned int cnt=0;
		FXuint pos=0;
		const FXint len=temp.length();
		const FXchar* s=temp.text();
		while (pos<len) {
			// Convert Mac and Windows EOLs to Unix
			if (s[pos]=='\r')
				temp.replace(pos, (s[pos+1]=='\n')?'\0':'\n');
			//Count rows and columns
			if (s[pos]=='\n') {
				rows++;
				if (cnt>cols)
					cols=cnt;
				cnt=0;
			} else
				cnt++;
			pos+=FXString::utfBytes[(FXuchar)s[pos]];
		}
		if (cnt>cols)
			cols=cnt;

		FXTRACE((10, "Rows: %d\n", rows));
		FXTRACE((10, "Cols: %d\n", cols));

		if (rows>1) {
			_font=new FXFont(getApp(), "courier", 10,
				FXFont::Normal, FXFont::Straight,
				FONTENCODING_DEFAULT, FXFont::NonExpanded, FXFont::Fixed);
			FXHorizontalFrame *frame=new FXHorizontalFrame(this, LAYOUT_FILL|FRAME_NORMAL, 0,0,0,0, 0,0,0,0, 0,0);
			FXText *text=new FXText(frame, 0, 0, TEXT_READONLY|TEXT_WORDWRAP|LAYOUT_FILL);
			text->setFont(_font);
			text->setText(temp);
			text->setVisibleRows((rows<20)?rows:20);
			text->setVisibleColumns((cols<80)?cols+2:80);
		} else {
			const FXString info=i->getKey(j);
			if (compare(info, "location")) { // FIXME
				//info.append(':');
				FXLabel *lbl=new FXLabel(matrix, tr(info.text()), 0, LAYOUT_FILL_X|JUSTIFY_RIGHT);
				lbl->setTextColor(lblColor);

				new textLabel(matrix, temp, LABEL_NORMAL, 32);
			}
		}
	}

	// Display album art
	FXGUISignal* imgLoaded=new FXGUISignal(getApp(), this, infoDialog::ID_IMG_LOADED);
	_imageLoader=new imageLoader(imgLoaded);

	FXString *_files;
	folder=FXPath::directory(folder);
	FXint n=FXDir::listFiles(_files, folder, "(cover|front|folder).*", FXDir::CaseFold);

	if (n) {
		FXString file=FXStringFormat("%s%c%s", folder.text(), PATHSEP, _files[0].text());

		FXTRACE((10, "Trying to load album art: %s\n", file.text()));
		_imageLoader->load(file);
		getApp()->beginWaitCursor();
	} else {
		_imgFrame->hide();
	}

	delete [] _files;

	if (!matrix->getFirst())
		new FXLabel(matrix, tr("No info"), 0, LAYOUT_FILL_X);
}

infoDialog::~infoDialog() {

	if (_imageLoader) {
		_imageLoader->join();
		delete _imageLoader;
		getApp()->endWaitCursor();
	}

	if (_imgFrame) {
		FXImage *img=_imgFrame->getImage();
		delete img;
	}
	delete _font;
}

long infoDialog::onCmdImgLoaded(FXObject*, FXSelector, void* ptr) {

	_imageLoader->join();

	FXImage *img=_imgFrame->getImage();
	_imgFrame->setImage(0);
	delete img;

	img=(FXImage*)ptr;

	if (img) {
		img->create();
		_imgFrame->setImage(img);
	} else {
		_imgFrame->hide();
	}

	getApp()->endWaitCursor();
	delPtr(_imageLoader);

	return 1;
}

==> fap_0.8.8.orig.tar.bz2: fap-0.8.8/src/gui/infoDialog.cpp <==
/*
 *  Copyright (C) 2006-2011 Leandro Nini
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
 */

#include "infoDialog.h"
#include "textLabel.h"
#include "settings.h"
#include "iconFactory.h"
#include "utils.h"

#if FOX_LEVEL<32
#  include <FXJPGImage.h>
#endif

#define IMAGESIZE 150

FXint imageLoader::run() {

	FXIconSource iconsource(FXApp::instance());
	FXImage *img;

#if FOX_LEVEL<32
	// Hack to overcome a FOX bug
	if (FXJPGImage::supported && !comparecase(FXPath::extension(_name), "jpeg"))
		img=iconsource.loadScaledImageFile(_name, IMAGESIZE, 0, FXJPGImage::fileExt);
	else
#endif
	img=iconsource.loadScaledImageFile(_name, IMAGESIZE, SETTINGS->albumArtQuality());

	_loaded->setData((void*)(FXival)img);
	_loaded->signal();

	return 0;
}

FXbool imageLoader::load(const FXString& file) {

	_name=file;

	return start();
}

/*****************************************************************/

FXDEFMAP(infoDialog) infoDialogMap[] = {
	// Message_Type			ID				Message_Handler
	FXMAPFUNC(SEL_IO_READ,		infoDialog::ID_IMG_LOADED,	infoDialog::onCmdImgLoaded),
};

FXIMPLEMENT(infoDialog, FXDialogBox, infoDialogMap, ARRAYNUMBER(infoDialogMap))

infoDialog::infoDialog(FXWindow* win, const metaData* i) :
	FXDialogBox(win, FXString::null, DECOR_TITLE|DECOR_BORDER),
	_font(0),
	_imageLoader(0),
	_imgFrame(0) {

	setTitle(tr("Song Info"));

	new FXButton(this, tr("&Close"), GET_ICON(icon_dialogcancel), this, FXDialogBox::ID_CANCEL,
		BUTTON_INITIAL|BUTTON_DEFAULT|BUTTON_NORMAL|LAYOUT_CENTER_X|LAYOUT_SIDE_BOTTOM);
	new FXSeparator(this, SEPARATOR_GROOVE|LAYOUT_FILL_X|LAYOUT_SIDE_BOTTOM);

	FXString folder=i->getInfo(metaData::LOCATION);
	if (folder.empty()) {
		new FXLabel(this, tr("No song loaded"));
		return;
	}

	FXHorizontalFrame *container=new FXHorizontalFrame(this, LAYOUT_FILL_X, 0,0,0,0, 0,0,0,0, 0,0);

	_imgFrame=new FXImageFrame(container, 0, LAYOUT_FIX_WIDTH|LAYOUT_FIX_HEIGHT|FRAME_SUNKEN|FRAME_THICK, 0, 0, IMAGESIZE, IMAGESIZE);

	FXMatrix* matrix=new FXMatrix(container, 2, MATRIX_BY_COLUMNS|LAYOUT_FILL_X);

	FXColor lblColor=altColor(getApp()->getForeColor());

	int j=-1;
	while ((j=i->moreInfo(j))>=0) {
		FXString temp=i->getInfo(j);
		if (temp.empty())
			continue;

		unsigned int rows=1;
		unsigned int cols=0;
		unsigned int cnt=0;
		FXuint pos=0;
		const FXint len=temp.length();
		const FXchar* s=temp.text();
		while (pos<len) {
			// Convert Mac and Windows EOLs to Unix
			if (s[pos]=='\r')
				temp.replace(pos, (s[pos+1]=='\n')?'\0':'\n');
			//Count rows and columns
			if (s[pos]=='\n') {
				rows++;
				if (cnt>cols)
					cols=cnt;
				cnt=0;
			} else
				cnt++;
			pos+=FXString::utfBytes[(FXuchar)s[pos]];
		}
		if (cnt>cols)
			cols=cnt;

		FXTRACE((10, "Rows: %d\n", rows));
		FXTRACE((10, "Cols: %d\n", cols));

		if (rows>1) {
			_font=new FXFont(getApp(), "courier", 10,
				FXFont::Normal, FXFont::Straight,
				FONTENCODING_DEFAULT, FXFont::NonExpanded, FXFont::Fixed);
			FXHorizontalFrame *frame=new FXHorizontalFrame(this, LAYOUT_FILL|FRAME_NORMAL, 0,0,0,0, 0,0,0,0, 0,0);
			FXText *text=new FXText(frame, 0, 0, TEXT_READONLY|TEXT_WORDWRAP|LAYOUT_FILL);
			text->setFont(_font);
			text->setText(temp);
			text->setVisibleRows((rows<20)?rows:20);
			text->setVisibleColumns((cols<80)?cols+2:80);
		} else {
			const FXString info=i->getKey(j);
			if (compare(info, "location")) { // FIXME
				//info.append(':');
				FXLabel *lbl=new FXLabel(matrix, tr(info.text()), 0, LAYOUT_FILL_X|JUSTIFY_RIGHT);
				lbl->setTextColor(lblColor);

				new textLabel(matrix, temp, JUSTIFY_LEFT, 32);
			}
		}
	}

	// Display album art
	FXGUISignal* imgLoaded=new FXGUISignal(getApp(), this, infoDialog::ID_IMG_LOADED);
	_imageLoader=new imageLoader(imgLoaded);

	FXString *_files;
	folder=FXPath::directory(folder);
	FXint n=FXDir::listFiles(_files, folder, "(cover|front|folder).*", FXDir::CaseFold);

	if (n) {
		FXString file=FXStringFormat("%s%c%s", folder.text(), PATHSEP, _files[0].text());

		FXTRACE((10, "Trying to load album art: %s\n", file.text()));
		_imageLoader->load(file);
		getApp()->beginWaitCursor();
	} else {
		_imgFrame->hide();
	}

	delete [] _files;

	if (!matrix->getFirst())
		new FXLabel(matrix, tr("No info"), 0, LAYOUT_FILL_X);
}

infoDialog::~infoDialog() {

	if (_imageLoader) {
		_imageLoader->join();
		delete _imageLoader;
		getApp()->endWaitCursor();
	}

	if (_imgFrame) {
		FXImage *img=_imgFrame->getImage();
		delete img;
	}
	delete _font;
}

long infoDialog::onCmdImgLoaded(FXObject*, FXSelector, void* ptr) {

	_imageLoader->join();

	FXImage *img=_imgFrame->getImage();
	_imgFrame->setImage(0);
	delete img;

	img=(FXImage*)ptr;

	if (img) {
		img->create();
		_imgFrame->setImage(img);
	} else {
		_imgFrame->hide();
	}

	getApp()->endWaitCursor();
	delPtr(_imageLoader);

	return 1;
}
